/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.guest.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.modules.Module;
import com.openexchange.mailaccount.MailAccountStorageService;
import com.openexchange.session.Session;
import com.openexchange.share.ShareTarget;
import com.openexchange.share.ShareTargetPath;
import com.openexchange.share.core.HandlerParameters;
import com.openexchange.share.core.ModuleHandler;
import com.openexchange.share.groupware.TargetProxy;

/**
 * {@link GuardMailStorageHandler}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.10.0
 */
public class GuardMailStorageHandler implements ModuleHandler {

    MailAccountStorageService mass;
    GuardGuestUserService userService;

    public GuardMailStorageHandler(MailAccountStorageService mass, GuardGuestUserService userService) {
        this.mass = Objects.requireNonNull(mass, "Missing MailAccountStorageService");
        this.userService = Objects.requireNonNull(userService, "Missing Guest User service");
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleAdjuster#adjustTarget(com.openexchange.share.ShareTarget, com.openexchange.session.Session, int)
     */
    @Override
    public ShareTarget adjustTarget(ShareTarget target, Session session, int targetUserId) throws OXException {
        // TODO Auto-generated method stub
        return target;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleAdjuster#adjustTarget(com.openexchange.share.ShareTarget, int, int, int)
     */
    @Override
    public ShareTarget adjustTarget(ShareTarget target, int contextId, int requestUserId, int targetUserId) throws OXException {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.groupware.spi.ModuleExtension#getModules()
     */
    @Override
    public Collection<String> getModules() {
        //return Arrays.asList(Module.GUARD.getName());
        return Arrays.asList(Module.MAIL.getName());
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#loadTargets(java.util.List, com.openexchange.share.core.HandlerParameters)
     */
    @Override
    public List<TargetProxy> loadTargets(List<ShareTarget> targets, HandlerParameters parameters) throws OXException {
        return Arrays.asList(new GuardMailTargetProxy());
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#loadTarget(com.openexchange.share.ShareTarget, com.openexchange.session.Session)
     */
    @Override
    public TargetProxy loadTarget(ShareTarget target, Session session) throws OXException {
        // TODO Auto-generated method stub
        return new GuardMailTargetProxy();
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#loadTarget(java.lang.String, java.lang.String, com.openexchange.groupware.contexts.Context, int)
     */
    @Override
    public TargetProxy loadTarget(String folder, String item, Context context, int guestID) throws OXException {
        // TODO Auto-generated method stub
        return new GuardMailTargetProxy();
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#canShare(com.openexchange.share.groupware.TargetProxy, com.openexchange.share.core.HandlerParameters)
     */
    @Override
    public boolean canShare(TargetProxy proxy, HandlerParameters parameters) {
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#updateObjects(java.util.List, com.openexchange.share.core.HandlerParameters)
     */
    @Override
    public void updateObjects(List<TargetProxy> modified, HandlerParameters parameters) throws OXException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#touchObjects(java.util.List, com.openexchange.share.core.HandlerParameters)
     */
    @Override
    public void touchObjects(List<TargetProxy> touched, HandlerParameters parameters) throws OXException {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#isVisible(java.lang.String, java.lang.String, int, int)
     */
    @Override
    public boolean isVisible(String folder, String item, int contextID, int guestID) throws OXException {
        // TODO Auto-generated method stub
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#mayAdjust(com.openexchange.share.ShareTarget, com.openexchange.session.Session)
     */
    @Override
    public boolean mayAdjust(ShareTarget target, Session session) throws OXException {
        // TODO Auto-generated method stub
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#exists(java.lang.String, java.lang.String, int, int)
     */
    @Override
    public boolean exists(String folder, String item, int contextID, int guestID) throws OXException {
        // TODO Auto-generated method stub
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#listTargets(int, int)
     */
    @Override
    public List<TargetProxy> listTargets(int contextID, int guestID) throws OXException {
        if (hasTargets(contextID, guestID)) {
            return Arrays.asList(new GuardMailTargetProxy());
        }
        return Collections.emptyList();
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#hasTargets(int, int)
     */
    @Override
    public boolean hasTargets(int contextID, int guestID) throws OXException {
        if (!mass.existsMailAccount(0, guestID, contextID)) return false;  // If no mail account, not Guard share
        return userService.hasMailItems(guestID, contextID);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#getPath(com.openexchange.share.ShareTarget, com.openexchange.session.Session)
     */
    @Override
    public ShareTargetPath getPath(ShareTarget target, Session session) throws OXException {
        // TODO Auto-generated method stub
        return new ShareTargetPath(target.getModule(), target.getFolder(), target.getItem());
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.share.core.ModuleHandler#getPath(com.openexchange.share.ShareTarget, int, int)
     */
    @Override
    public ShareTargetPath getPath(ShareTarget target, int contextID, int guestID) throws OXException {
        // TODO Auto-generated method stub
        return null;
    }

}
