/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.guest.impl;

import java.util.AbstractMap.SimpleEntry;
import java.util.Arrays;
import java.util.List;
import org.slf4j.Logger;
import com.openexchange.config.ConfigurationService;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.container.FolderObject;
import com.openexchange.groupware.notify.hostname.HostData;
import com.openexchange.groupware.notify.hostname.HostnameService;
import com.openexchange.guard.guest.MailGuestService;
import com.openexchange.guard.guest.OxMailAccountCreatorService;
import com.openexchange.serverconfig.ServerConfigService;
import com.openexchange.session.Session;
import com.openexchange.share.CreatedShares;
import com.openexchange.share.ShareInfo;
import com.openexchange.share.ShareService;
import com.openexchange.share.ShareTarget;
import com.openexchange.share.recipient.GuestRecipient;


/**
 * {@link MailGuestServiceImpl}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.10.0
 */
public class MailGuestServiceImpl implements MailGuestService {

    private final OxMailAccountCreatorService accountCreator;
    private final ShareService shareService;
    private final ConfigurationService configurationService;
    private final ServerConfigService serverConfig;
    private static Logger logger = org.slf4j.LoggerFactory.getLogger(MailGuestServiceImpl.class);

    public MailGuestServiceImpl (OxMailAccountCreatorService accountCreator, ShareService shareService, ConfigurationService configurationService, ServerConfigService serverConfig) {
        this.accountCreator = accountCreator;
        this.shareService = shareService;
        this.configurationService = configurationService;
        this.serverConfig = serverConfig;
    }

    /**
     * Private function to check if language exists in backend translations
     * @param language
     * @return
     */
    private boolean languageExists(String language, Session session) {
        if (language == null) {
            return false;
        }
        try {
            com.openexchange.serverconfig.ServerConfig config = serverConfig.getServerConfig((String) session.getParameter(Session.PARAM_HOST_NAME), session);
            List<SimpleEntry<String, String>> avail = config.getLanguages();
            for (SimpleEntry<String, String> lang : avail) {
                if (lang.getKey().equals(language)) {
                    return true;
                }
            }
        } catch (OXException e) {
            logger.error("Error parsing available languages for Guest email ", e);
        }
        return false;
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.guest.MailGuestService#createGuestLink(java.lang.String, java.lang.String)
     */
    @Override
    public String createGuestLink(Session session, String email, String mailId, String language) throws OXException {
        GuestRecipient guestRecipient = new GuestRecipient();
        guestRecipient.setEmailAddress(email);
        if (!languageExists(language, session)) {
            language = null;  // Null language defaults to senders language
        }
        guestRecipient.setPreferredLanguage(language);
        ShareTarget shareTarget = new ShareTarget(FolderObject.MAIL, "default0/INBOX", mailId);
        CreatedShares shares = shareService.addTarget(session, shareTarget, Arrays.asList(guestRecipient));
        ShareInfo share = shares.getShare(guestRecipient);
        accountCreator.createAccount(share.getGuest(), session);
        HostData hostData = (HostData) session.getParameter(HostnameService.PARAM_HOST_DATA);
        if (hostData == null) {
            hostData = new GuestHostDataImpl(configurationService.getProperty("com.openexchange.dispatcher.prefix", "/ajax/"));  // Basically empty Host Data, will be filled with com.openexchange.share.guestHostname
        }
        String url = share.getShareURL(hostData);
        return url;
    }


}
