/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl.guests;

import java.net.URI;
import java.net.URISyntaxException;
import javax.mail.internet.idn.IDNA;
import com.openexchange.exception.OXException;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.mail.api.IMailProperties;
import com.openexchange.mail.api.MailCapabilities;
import com.openexchange.mail.api.MailConfig;
import com.openexchange.mail.api.UrlInfo;
import com.openexchange.tools.net.URIDefaults;
import com.openexchange.tools.net.URIParser;

/**
 * {@link GuardGuestMailConfig}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.10.0
 */
public class GuardGuestMailConfig extends MailConfig {

    private int port;
    private String server;
    private boolean secure;
    private IMailProperties mailProperties;

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#getCapabilities()
     */
    @Override
    public MailCapabilities getCapabilities() {
       return MailCapabilities.EMPTY_CAPS;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#getPort()
     */
    @Override
    public int getPort() {
        return this.port;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#getServer()
     */
    @Override
    public String getServer() {
        return server;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#isSecure()
     */
    @Override
    public boolean isSecure() {
        return this.secure;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#setPort(int)
     */
    @Override
    public void setPort(int port) {
        this.port = port;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#setSecure(boolean)
     */
    @Override
    public void setSecure(boolean secure) {
        this.secure = secure;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#setServer(java.lang.String)
     */
    @Override
    public void setServer(String server) {
        this.server = server == null ? null : IDNA.toUnicode(server);
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#getMailProperties()
     */
    @Override
    public IMailProperties getMailProperties() {
        return this.mailProperties;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#setMailProperties(com.openexchange.mail.api.IMailProperties)
     */
    @Override
    public void setMailProperties(IMailProperties mailProperties) {
        this.mailProperties = mailProperties;
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailConfig#parseServerURL(com.openexchange.mail.api.UrlInfo)
     */
    @Override
    protected void parseServerURL(UrlInfo urlInfo) throws OXException {
        final URI uri;
        try {
            uri = URIParser.parse(urlInfo.getServerURL(), URIDefaults.NULL);
        } catch (URISyntaxException e) {
            throw GuardExceptionCodes.URI_PARSE_FAILED.create(e, urlInfo.getServerURL());
        }
        this.server = uri.getHost();
        this.port = uri.getPort();
    }

}
