/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.sharing;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.file.storage.DefaultFileStorageGuestObjectPermission;
import com.openexchange.file.storage.DefaultFileStorageObjectPermission;
import com.openexchange.file.storage.FileStorageObjectPermission;
import com.openexchange.groupware.ComparedPermissions;
import com.openexchange.guard.api.GuardApi;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.file.storage.AbstractGuardInputStream;
import com.openexchange.guard.internal.authentication.AuthenticationTokenUtils;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.session.Session;
import com.openexchange.share.recipient.GuestRecipient;

/**
 * {@link GuardSharingModificationStream}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.10.0
 */
public class GuardSharingModificationStream extends AbstractGuardInputStream {

    private final GuardApi guardApi;
    private final GuardAuthenticationToken authentication;
    private final Session session;
    private InputStream originalFile;
    private List<FileStorageObjectPermission> newGuestPermissions;
    private Collection newPermissions;

    public GuardSharingModificationStream (GuardApi guardApi, InputStream originalFile, ComparedPermissions permissions, GuardAuthenticationToken authentication, Session session) {
        this.guardApi = guardApi;
        this.authentication = authentication;
        this.session = session;
        this.originalFile = originalFile;
        this.newGuestPermissions = null;
        this.newPermissions = permissions.getNewPermissions();
    }

    public GuardSharingModificationStream (GuardApi guardApi, InputStream originalFile, List<FileStorageObjectPermission> guestPermissions, GuardAuthenticationToken authentication, Session session) {
        this.guardApi = guardApi;
        this.authentication = authentication;
        this.session = session;
        this.originalFile = originalFile;
        this.newGuestPermissions = guestPermissions;
        this.newPermissions = null;
    }


    /**
     * Get list of email addresses of users being added that have not previously been created in OX
     * @param list
     * @return
     * @throws JSONException
     */
    private JSONArray getEmails (List<DefaultFileStorageGuestObjectPermission> list) throws JSONException {
        JSONArray items = new JSONArray();
        for (DefaultFileStorageGuestObjectPermission pem : list) {
            JSONObject user = new JSONObject();
            GuestRecipient recip = (GuestRecipient) pem.getRecipient();
            user.put("email", recip.getEmailAddress());
            items.put(user);
        }
        return items;
    }


    private JSONObject createJSON () throws JSONException {
        JSONObject data = AuthenticationTokenUtils.toJsonObject(authentication);
        data.put("updatedShares", newGuestPermissions == null ? SharingUtil.getIDs(newPermissions) : SharingUtil.getIDs(newGuestPermissions));
        data.put("contextId", session.getContextId());
        data.put("user_Id", session.getUserId());
        return data;
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.file.storage.AbstractGuardInputStream#getGuardStream()
     */
    @Override
    protected InputStream getGuardStream() throws OXException {
        JSONObject json;
        try {
            json = createJSON();
        } catch (JSONException e) {
            // Improve throw
            throw OXException.general("Unable to create JSON");
        }
        ByteArrayInputStream jsonStream = new ByteArrayInputStream(json.toString().getBytes());
        LinkedHashMap<String, InputStream> multipartParameters = new LinkedHashMap<>();
        multipartParameters.put("json", jsonStream);
        multipartParameters.put("data", originalFile);

        return guardApi.processResources(GuardApis.mapFor("action", "changeshare", "respondWithJSON", "true" /* forcing guard to render errors in JSON */), multipartParameters, null, session);
    }

}
